@@ -1,17 +1,24 @@
-from django.conf import settings
-from django.utils.importlib import import_module
-from django.conf.urls import patterns, url, include
-from django.test import TestCase
-from rest_framework_swagger.urlparser import UrlParser
-from rest_framework_swagger.docgenerator import DocumentationGenerator
-from django.contrib.admindocs.utils import trim_docstring
+from unipath import Path
 
+from django.conf import settings
+from django.conf.urls import patterns, url, include
+from django.contrib.auth.models import User
+from django.contrib.admindocs.utils import trim_docstring
+from django.http import HttpRequest
+from django.test import TestCase
+from django.utils.importlib import import_module
 from django.views.generic import View
+
+from rest_framework.decorators import action
 from rest_framework.views import APIView
 from rest_framework.generics import ListCreateAPIView
 from rest_framework import serializers
 from rest_framework.routers import DefaultRouter
-from rest_framework.viewsets import ModelViewSet
+from rest_framework.viewsets import ModelViewSet, ViewSet
+
+from .urlparser import UrlParser
+from .docgenerator import DocumentationGenerator
+from .introspectors import ViewSetIntrospector, APIViewIntrospector
 
 
 class MockApiView(APIView):
@@ -27,6 +34,7 @@ class MockApiView(APIView):
         pass
     pass
 
+
 class NonApiView(View):
     pass
 
@@ -63,9 +71,7 @@ class UrlParserTest(TestCase):
         self.assertEqual(len(self.url_patterns), len(apis))
 
     def test_flatten_url_tree_url_import(self):
-        urls = patterns('',
-            url(r'api/base/path/', include(self.url_patterns))
-        )
+        urls = patterns('', url(r'api/base/path/', include(self.url_patterns)))
         urlparser = UrlParser()
         apis = urlparser.get_apis(urls)
 
@@ -87,7 +93,6 @@ class UrlParserTest(TestCase):
         self.assertEqual([], apis)
 
     def test_flatten_url_tree_url_import_with_routers(self):
-        from django.contrib.auth.models import User
 
         class MockApiViewSet(ModelViewSet):
             serializer_class = CommentSerializer
@@ -101,9 +106,7 @@ class UrlParserTest(TestCase):
         router.register(r'other_views', MockApiViewSet)
         router.register(r'more_views', MockApiViewSet)
 
-        urls_app = patterns('',
-            url(r'^', include(router.urls))
-        )
+        urls_app = patterns('', url(r'^', include(router.urls)))
         urls = patterns('',
             url(r'api/', include(urls_app)),
             url(r'test/', include(urls_app))
@@ -135,7 +138,6 @@ class UrlParserTest(TestCase):
 
         self.assertEqual(4, len(apis))
 
-
     def test_assemble_endpoint_data(self):
         """
         Tests that the endpoint data is correctly packaged
@@ -158,8 +160,6 @@ class UrlParserTest(TestCase):
         self.assertIsNone(data)
 
     def test_exclude_router_api_root(self):
-        from django.contrib.auth.models import User
-
         class MyViewSet(ModelViewSet):
             serializer_class = CommentSerializer
             model = User
@@ -175,8 +175,6 @@ class UrlParserTest(TestCase):
         self.assertEqual(4, urls_created - len(apis))
 
 
-
-
 class DocumentationGeneratorTest(TestCase):
     def setUp(self):
         self.url_patterns = patterns('',
@@ -188,7 +186,7 @@ class DocumentationGeneratorTest(TestCase):
 
     def test_get_description(self):
         generator = DocumentationGenerator()
-        description = generator.__get_description__(MockApiView())
+        description = generator.get_description(MockApiView())
 
         self.assertEqual('A Test View', description)
 
@@ -201,12 +199,12 @@ class DocumentationGeneratorTest(TestCase):
                 """
             pass
 
-        generator = DocumentationGenerator()
-        docs_get = generator.__get_method_docs__(TestApiView(), 'GET')
+        introspector = APIViewIntrospector()
+        docs_get = introspector.get_method_docs(TestApiView, 'GET')
 
-        self.assertEqual("Here are my comments", docs_get)
+        self.assertEqual("Here are my comments", docs_get.strip())
 
-    def test_get_method_generic_api_view(self):
+    def test_get_method_summary_without_docstring(self):
 
         class MyListView(ListCreateAPIView):
             """
@@ -214,8 +212,8 @@ class DocumentationGeneratorTest(TestCase):
             """
             pass
 
-        generator = DocumentationGenerator()
-        method_docs = generator.__get_method_docs__(MyListView(), 'POST')
+        introspector = APIViewIntrospector()
+        method_docs = introspector.get_method_summary(MyListView, 'POST')
 
         self.assertEqual("My comment", method_docs)
 
@@ -231,7 +229,7 @@ class DocumentationGeneratorTest(TestCase):
             'pattern': patterns('')
         }
         docgen = DocumentationGenerator()
-        operations = docgen.__get_operations__(api)
+        operations = docgen.get_operations(api)
 
         self.assertEqual('POST', operations[0]['httpMethod'])
 
@@ -246,7 +244,7 @@ class DocumentationGeneratorTest(TestCase):
             'pattern': patterns('')
         }
         docgen = DocumentationGenerator()
-        operations = docgen.__get_operations__(api)
+        operations = docgen.get_operations(api)
 
         self.assertEqual([], operations)
 
@@ -260,7 +258,7 @@ class DocumentationGeneratorTest(TestCase):
             pass
 
         docgen = DocumentationGenerator()
-        docstring = docgen.__strip_params_from_docstring__(trim_docstring(AnAPIView.__doc__))
+        docstring = docgen.strip_params_from_docstring(trim_docstring(AnAPIView.__doc__))
 
         self.assertEqual("My comments are here<br/>", docstring)
 
@@ -281,7 +279,7 @@ class DocumentationGeneratorTest(TestCase):
             pass
 
         docgen = DocumentationGenerator()
-        docstring = docgen.__strip_params_from_docstring__(TestView.__doc__)
+        docstring = docgen.strip_params_from_docstring(TestView.__doc__)
         expected = 'Creates a new user.<br/>Returns: token - auth token<br/>'
 
         self.assertEqual(expected, docstring)
@@ -308,38 +306,51 @@ class DocumentationGeneratorTest(TestCase):
         apis = urlparser.get_apis(url_patterns)
 
         docgen = DocumentationGenerator()
-        serializers = docgen.__get_serializer_set__(apis)
+        serializers = docgen.get_serializer_set(apis)
 
         self.assertIn(CommentSerializer, serializers)
 
     def test_get_serializer_fields(self):
         docgen = DocumentationGenerator()
-        fields = docgen.__get_serializer_fields__(CommentSerializer)
+        fields = docgen.get_serializer_fields(CommentSerializer)
 
         self.assertEqual(3, len(fields))
 
     def test_get_serializer_fields_api_with_no_serializer(self):
         docgen = DocumentationGenerator()
-        fields = docgen.__get_serializer_fields__(None)
+        fields = docgen.get_serializer_fields(None)
 
         self.assertIsNone(fields)
 
+    def test_get_serializer_class_access_request_context(self):
+        class MyListView(ListCreateAPIView):
+            serializer_class = CommentSerializer
+            def get_serializer_class(self):
+                self.serializer_class.context = {'request': self.request}
+                return self.serializer_class
+
+        docgen = DocumentationGenerator()
+        callback = MyListView
+        callback.request = HttpRequest()
+        serializer_class = docgen.get_serializer_class(MyListView)
+
+        self.assertIs(serializer_class, CommentSerializer)
+
     def test_build_body_parameters(self):
         class SerializedAPI(ListCreateAPIView):
             serializer_class = CommentSerializer
 
         docgen = DocumentationGenerator()
-        params = docgen.__build_body_parameters__(SerializedAPI)
+        params = docgen.build_body_parameters(SerializedAPI)
 
         self.assertEqual('CommentSerializer', params['name'])
 
-
     def test_build_form_parameters(self):
         class SerializedAPI(ListCreateAPIView):
             serializer_class = CommentSerializer
 
         docgen = DocumentationGenerator()
-        params = docgen.__build_form_parameters__(SerializedAPI, 'POST')
+        params = docgen.build_form_parameters(SerializedAPI, 'POST')
 
         self.assertEqual(len(CommentSerializer().get_fields()), len(params))
 
@@ -353,7 +364,7 @@ class DocumentationGeneratorTest(TestCase):
             serializer_class = MySerializer
 
         docgen = DocumentationGenerator()
-        params = docgen.__build_form_parameters__(MyAPIView, 'POST')
+        params = docgen.build_form_parameters(MyAPIView, 'POST')
 
         self.assertEqual(1, len(params))  # Read only field is ignored
         param = params[0]
@@ -365,29 +376,64 @@ class DocumentationGeneratorTest(TestCase):
         self.assertEqual(10, param['allowableValues']['min'])
         self.assertEqual('Vandalay Industries', param['defaultValue'])
 
+
+class ViewTestIntrospectorTest(TestCase):
     def test_get_allowed_methods(self):
         """
         Tests a ModelViewSet's allowed methods. If the path includes something like {pk},
         consider it an object view, otherwise, a list view
         """
-        from django.contrib.auth.models import User
 
         class MyViewSet(ModelViewSet):
             serializer_class = CommentSerializer
             model = User
 
-        docgen = DocumentationGenerator()
+        introspector = ViewSetIntrospector()
 
         # Test a list endpoint
-        allowed_methods = docgen.__get_allowed_methods__(MyViewSet, '/api/endpoint')
+        allowed_methods = introspector.get_allowed_methods(MyViewSet, '/api/endpoint')
         self.assertEqual(2, len(allowed_methods))
         self.assertIn('POST', allowed_methods)
         self.assertIn('GET', allowed_methods)
 
         # Test an object endpoint
-        allowed_methods = docgen.__get_allowed_methods__(MyViewSet, '/api/endpoint/{pk}')
+        allowed_methods = introspector.get_allowed_methods(MyViewSet, '/api/endpoint/{pk}')
         self.assertEqual(4, len(allowed_methods))
         self.assertIn('POST', allowed_methods)
         self.assertIn('PATCH', allowed_methods)
         self.assertIn('DELETE', allowed_methods)
         self.assertIn('GET', allowed_methods)
+
+    def test_is_custom_action_true(self):
+        class MyViewSet(ViewSet):
+            @action()
+            def my_action(self, request):
+                pass
+
+        path = '/api/{pk}/my_action/'
+        introspector = ViewSetIntrospector()
+
+        self.assertTrue(introspector.is_custom_action(MyViewSet, path))
+
+    def test_viewset_action_allowed_methods(self):
+        class MyViewSet(ViewSet):
+            @action()
+            def do_something(self, request):
+                """
+                This is a REST action that does something. Django REST maps it
+                to an HTTP POST method
+                """
+
+        callback = MyViewSet
+        path = '/api-path/myviewset/{pk}/do_something/'
+
+        path_components = Path(path).components()
+        last_element = unicode(path_components[len(path_components) - 1])
+
+        action_func = eval('callback.%s.im_func' % last_element)
+        expected_result = action_func.bind_to_methods
+
+        introspector = ViewSetIntrospector()
+        allowed_methods = introspector.get_allowed_methods(callback, path)
+
+        self.assertEqual(expected_result, allowed_methods)
